/*

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

package settings

import (
	"bytes"
	"reflect"
	"text/template"

	"github.com/presslabs/controller-util/rand"
	corev1 "k8s.io/api/core/v1"
)

func (p *Parameters) InitValues() {
	p.InitRandValues()
	p.InitTemplateValues(p.KeyPairValues())
}

func (p *Parameters) InitRandValues() {
	for _, param := range *p {

		if len(param.Key) > 0 && len(param.Value) == 0 && param.Generate != GenerateTemplate && param.Type != ObjectFieldParameter {
			var size int
			switch param.Generate {
			case GenerateRand24:
				size = 24
				random, err := rand.AlphaNumericString(size)
				if err != nil {
				}
				param.Value = random

			case GenerateRand12:
				size = 12
				random, err := rand.AlphaNumericString(size)
				if err != nil {
				}
				param.Value = random
			}
		}
	}
}

func (p *Parameters) InitTemplateValues(values map[string]string) {

	for _, param := range *p {
		if param.Generate == GenerateTemplate {
			tmpl, err := template.New(param.Key).Parse(param.Value)

			if err != nil {
				//	return nil, err
			}

			var tpl bytes.Buffer
			err = tmpl.Execute(&tpl, values)

			if err != nil {
				//	return nil, err
			}

			param.Value = tpl.String()
		}
	}
}

func (p *Parameters) SortByMountType() map[MountType]Parameters {
	data := make(map[MountType]Parameters)

	for _, param := range *p {
		data[param.MountType] = append(data[param.MountType], param)
	}

	return data
}

func (p *Parameters) KeyPairValues() map[string]string {
	data := make(map[string]string)

	sorted := p.SortByMountType()

	for _, param := range sorted[MountLiteral] {

		if len(param.Value) > 0 && len(param.Key) > 0 {
			data[param.Key] = param.Value
		}

	}

	for _, param := range sorted[MountFrom] {

		if len(param.Value) > 0 && len(param.Key) > 0 {
			data[param.Key] = param.Value
		}
	}

	return data
}

func (p *Parameters) GetData() map[string]string {
	data := make(map[string]string)

	for _, param := range *p {
		if param.MountType == MountFrom && param.Type != ObjectFieldParameter && len(param.Value) > 0 {
			data[param.Key] = param.Value
		}
	}

	return data
}

func (p *Parameters) GetSecretData() map[string]string {
	data := make(map[string]string)

	for _, param := range *p {
		if param.MountType == MountFrom && param.Type == SecretParameter {
			data[param.Key] = param.Value
		}
	}

	return data
}

func (p *Parameters) GetConfigData() map[string]string {
	data := make(map[string]string)

	for _, param := range *p {
		if param.MountType == MountFrom &&
			param.Type == ConfigParameter &&
			len(param.Value) > 0 {
			data[param.Key] = param.Value
		}
	}

	return data
}

func (p *Parameters) GetEnv() []corev1.EnvVar {
	envVars := []corev1.EnvVar{}
	envVar := corev1.EnvVar{}

	var err error

	for _, param := range *p {
		envVar, err = param.GetEnvVar()

		if err != nil {
		}

		if len(envVar.Name) > 0 {
			envVars = append(envVars, envVar)
		}
	}

	// TODO IMPROVE
	if len(envVars) > 0 {
		return envVars
	}
	return nil
}

func Marshal(i interface{}) (Parameters, error) {
	parameters := Parameters{}

	t := reflect.TypeOf(i)

	ifv := reflect.ValueOf(i)

	for ii := 0; ii < t.NumField(); ii++ {
		f := t.Field(ii)

		fv := ifv.Field(ii)

		parameter, ok := fv.Interface().(Parameter)
		if !ok {
			//	return nil, ok
		}

		env, ok := f.Tag.Lookup("env")
		if !ok {
			//	return nil, ok
		}

		if len(parameter.Key) == 0 {
			parameter.Key = env
		}

		parameters = append(parameters, &parameter)
	}

	return parameters, nil
}
